// Copyright (c) 2012, Marco Muccinelli
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
// * Neither the name of the <organization> nor the
// names of its contributors may be used to endorse or promote products
// derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <UIKit/UIKit.h>
#import <MUKMediaGallery/MUKMediaAssetProtocol.h>

/**
 A view which displays a paginated carousel of medias.
 */
@class MUKImageFetcher, MUKURLConnection;
@protocol MUKMediaImageAsset;
@interface MUKMediaCarouselView : UIView
/** @name Properties */
/**
 Media assets.
 
 This an array of instances which conform to MUKMediaAsset protocol.
 */
@property (nonatomic, strong) NSArray *mediaAssets;
/**
 Image fetcher which loads and stores thumbnails (only in memory).
 
 This object is lazy loaded, but you can customize its behaviour.
 
 You could also reuse a cache (e.g. reusing an already populated 
 MUKMediaThumbnailsView's cache).
 
 @warning You can not set [MUKImageFetcher shouldStartConnectionHandler] 
 because it is used internally not to load invisible thumbnails.
 */
@property (nonatomic, strong, readonly) MUKImageFetcher *thumbnailsFetcher;
/**
 Image fetcher which loads and stores full images.
 
 This object is lazy loaded, but you can customize its behaviour.
 
 @warning You should set file cache handlers (on `imagesFetcher.cache`)
 if you decide to cache images to file. Mind that cache key is 
 [MUKMediaAsset mediaURL].
 @warning You can not set [MUKImageFetcher shouldStartConnectionHandler] 
 because it is used internally not to load invisible images.
 */
@property (nonatomic, strong, readonly) MUKImageFetcher *imagesFetcher;
/**
 Cache images to memory.
 
 Default is `NO`.
 
 @warning Memory cache is disabled by default because RAM usage may
 become expensive with big images. If you know you images are not too big,
 you could enable memory cache to make transition more fluid. Mind that cache 
 key is [MUKMediaAsset mediaURL].
 */
@property (nonatomic) BOOL usesImageMemoryCache;
/**
 Cache images to file.
 
 Default is `YES` and so images are searched/saved in file cache too 
 by default.
 
 @warning You should set file cache handlers (on `imagesFetcher.cache`)
 if you decide to cache images to file. Mind that cache key is 
 [MUKMediaAsset mediaURL].
 */
@property (nonatomic) BOOL usesImageFileCache;
/**
 Cleans images memory cache when reloadMedias is called.
 
 Default is `YES`.
 */
@property (nonatomic) BOOL purgesImagesMemoryCacheWhenReloading;
/**
 Space between medias.
 
 Default is `20.0f`.
 */
@property (nonatomic) CGFloat mediaOffset;
/**
 Minimum zoom scale for an image.
 
 Default is `1.0f`
 */
@property (nonatomic) float imageMinimumZoomScale;
/**
 Maximum zoom scale for an image.
 
 Default is `3.0f`
 */
@property (nonatomic) float imageMaximumZoomScale;
/**
 Autoplays audio and video medias.
 
 Default is `NO`.
 */
@property (nonatomic) BOOL autoplaysMedias;
/**
 Show and hide overlay view on user touches.
 
 Default is `YES`.
 */
@property (nonatomic) BOOL togglesOverlayViewOnUserTouch;

/** @name Handlers */
/**
 Handler called at every scroll step.
 
 @warning This handler is called only if scrolling is animated
 or generated by an user touch.
 */
@property (nonatomic, copy) void (^scrollHandler)(void);
/**
 Handler called as scroll finishes.
 */
@property (nonatomic, copy) void (^scrollCompletionHandler)(void);
/**
 Handler called when a media is tapped once.
 */
@property (nonatomic, copy) void (^mediaAssetTappedHandler)(NSInteger index);
/**
 Handler called when a media is zoomed (at every zoom step).
 */
@property (nonatomic, copy) void (^mediaAssetZoomedHandler)(NSInteger index, float scale);
/**
 Handler called to create a connection used to download full asset image.
 
 Return `nil` if you want to use a standard connection.
 
 @warning [MUKURLConnection userInfo] will be overwritten with `mediaImageAsset`.
 */
@property (nonatomic, copy) MUKURLConnection* (^imageConnectionHandler)(id<MUKMediaImageAsset> mediaImageAsset, NSInteger index);
/**
 Handler called after a media asset has been displayed into a cell.
 
 This handler is invoked after spinner has been hidden and overlay view
 is forced to visible (if needed).
 */
@property (nonatomic, copy) void (^mediaAssetDisplayedHandler)(id<MUKMediaAsset> mediaAsset, NSInteger index);

/** @name Methods */
/**
 Reload media views.
 */
- (void)reloadMedias;
/**
 Currently displayed media index.
 
 @return Index of displayed media. If there aren't media assets,
 it returns `NSNotFound`.
 */
- (NSInteger)currentMediaAssetIndex;
/**
 Scrolls to media asset.
 
 @param index Index of media asset to show.
 @param animated `YES` if you want to animate transition.
 */
- (void)scrollToMediaAssetAtIndex:(NSInteger)index animated:(BOOL)animated;
@end


@interface MUKMediaCarouselView (Callbacks)
/**
 Callback invoked when a media asset is tapped.
 
 Default implementation calls mediaAssetTappedHandler and, then,
 toggles overlay view (if togglesOverlayViewOnUserTouch is set to `YES`).
 
 @param index Media asset index.
 */
- (void)didTapMediaAssetAtIndex:(NSInteger)index;
/**
 Callback invoked when scroll to a media asset is completed.
 
 Default implementation shows media for current index and it calls
 scrollCompletionHandler if transition has been animated.
 
 At the end, if canHideOverlayViewAtIndex: returns `NO`, overlay view is
 hidden.
 
 @param index Media asset shown at the end of the scroll transition.
 @param animated `YES` if scroll has been animated.
 */
- (void)didScrollToMediaAssetAtIndex:(NSInteger)index animated:(BOOL)animated;
@end


@interface MUKMediaCarouselView (OverlayView)
/**
 Check conditions because overlay view should be shown.
 
 @param index Index of media asset.
 @return `YES` if overlay view should be shown.
 */
- (BOOL)shouldShowOverlayViewAtIndex:(NSInteger)index;
/**
 Check condition because overlay view can be hidden.
 
 @param index Index of media asset.
 @return `NO` if there is no cell at given index or if cell is displaying
 YouTube video with a `UIWebView` instance (it means taps can not be
 processed by cells, so overlay view visibily could not be toggled by
 user).
 */
- (BOOL)canHideOverlayViewAtIndex:(NSInteger)index;
/**
 Overlay view is hidden or not.
 
 By default overlay view is shown, so this method returns `NO`.
 
 @return `YES` if overlay view is hidden.
 */
- (BOOL)isOverlayViewHidden;
/**
 Hides or shows overlay view.
 
 @param hidden `YES` if you want to hide overlay view.
 @param animated `YES` if you want the transition to be animated.
 */
- (void)setOverlayViewHidden:(BOOL)hidden animated:(BOOL)animated;
/**
 Current overlay view insets.
 
 @return Current overlay view insets.
 */
- (UIEdgeInsets)overlayViewInsets;
/**
 Set current overlay view insets.
 
 @param insets Insets which will be set.
 @param animated `YES` if new insets should be applied with an animation.
 */
- (void)setOverlayViewInsets:(UIEdgeInsets)insets animated:(BOOL)animated;
@end
